@@ -52,6 +52,7 @@ import pandas as pd
 import pickle
 
 set_print_limit = False  # 显示有限的输出设置
+
 _SeriesType = Union[dict, tuple, list]
 
 _SeriesTypeBase = [tuple, list]
@@ -320,9 +321,8 @@ class Series:
 
     def _operator(self, sql):
         _axis_value = np.where(sql)[0]
-        return Series(
-            _axis_value,
-            self.name, self.unique, 'loc.operator')
+
+        return Series(_axis_value, self.name, self.unique, 'loc.operator')
 
     #
     # def __contains__(self, other: list):
@@ -526,7 +526,7 @@ class Matrix:
                 raise ValueError(f"Error param length: {axis_one}, it's length should be 3")
         else:
             raise ValueError(
-                f'Error type param: {axis_one, axis_two}, it should double int or str or axis_one is list and axis_two is None ')
+                f'Error type param: {axis_one, axis_two}, it should be double, int or str or axis_one is list and axis_two is None ')
 
         self.data = self.data.swapaxes(axis_one_num, axis_two_num)
 
@@ -684,6 +684,29 @@ class Matrix:
         #
         pass
 
+    def fill_na(self, method):
+        # 沿着第一轴（通常认为是时间轴）前向或者后向填充空值
+        # method 方法：前向，后向，先前向再后向，先后向再前向
+        self.T(0, 2)
+        if method == 'ffill':
+            for i in range(len(self.data)):
+                self.data[:, i, :] = ffill(self.data[:, i, :])
+        elif method == 'bfill':
+            for i in range(len(self.data)):
+                self.data[:, i, :] = bfill(self.data[:, i, :])
+        elif method == 'bffill':
+            for i in range(len(self.data)):
+                self.data[:, i, :] = bfill(self.data[:, i, :])
+                self.data[:, i, :] = ffill(self.data[:, i, :])
+        elif method == 'fbfill':
+            for i in range(len(self.data)):
+                self.data[:, i, :] = ffill(self.data[:, i, :])
+                self.data[:, i, :] = bfill(self.data[:, i, :])
+        else:
+            raise ValueError(f'Error method: {method}')
+        self.T(0, 2)
+        return self
+
     def sort(self, sort_axis_name: Union[str, int], assign_list: list = None, reverse=False):
         # 根据axis_name对应的轴值，进行二维矩阵的排序,可以根据指定的顺序列表assign_list（代替轴值）进行排序，
         # 可配合Series和features进行元素值排序，如根据某一天datetime的features.close的大小进行排序
@@ -789,12 +812,9 @@ class Matrix:
                         return Matrix(_data, _axis_name, unique=self.unique)
 
                     elif item.dtype == 'loc.and_series':
-                        # 特别注意：如果multi_series里有返回dataframe的，必须要放在最后一个
-                        # 如'bool'类型和'loc.unite'类型
 
                         new_matrix = self
                         for _series in item.values:
-                            # print(new_matrix)
                             if new_matrix.values.any():
                                 new_matrix = new_matrix[_series]
                             else:
@@ -817,24 +837,15 @@ class Matrix:
                             new_matrix = new_matrix[_series]
                             if new_matrix.values.any():
                                 return new_matrix
-
                 elif type(item) == slice:
-
                     _data = self.values[item]
-                    item = 0
-                    _axis_name = deepcopy(self.axis_name)
-                    _axis_name.pop(self._dimension_num_dict[item])
-                    i = 0
-                    _df_name = {'index': [], 'columns': []}
-                    for key, value in _axis_name.items():
-                        if i == 0:
-                            _df_name['index'] = value
-                            i += 1
-                        else:
-                            _df_name['columns'] = value
 
-                    return DataFrame(_data[0], _df_name, name=str(self._dimension_num_dict[item]) + '==' + str(
-                        self.axis_name[self._dimension_num_dict[item]][0]))
+                    _axis_name = deepcopy(self.axis_name)
+                    _axis_name_key = self._dimension_num_dict[0]
+                    _axis_name[_axis_name_key] = list(np.array(_axis_name[_axis_name_key])[item])
+
+                    return Matrix(_data, _axis_name, self.unique)
+
                 elif type(item) == int:
 
                     _data = self.values[item]
@@ -875,12 +886,11 @@ class Matrix:
                     elif type(item[0][1]) == int:
                         select_list = item[0][1]
                     else:
-                        _axis_value = self.axis_name[_name]
-                        axis_value_list = []
-                        _res = np.where(np.array(_axis_value) == item[0][1])[0][0]
-                        axis_value_list.append(_res)
-                        select_list = axis_value_list
-
+                        # self._dimension_dict
+                        # self._dimension_num_dict
+                        # self._axis_name_dict
+                        # self._axis_name_key
+                        raise ValueError(f'Error type item{type(item[0][1])}')
                     if _dimension == 0:
                         _value = self.values[select_list, :, :]
                     elif _dimension == 1:
@@ -969,12 +979,14 @@ class Matrix:
                         i += 1
 
                     return Matrix(_value, _axis_name, unique=self.unique)
+
                 else:
                     raise ValueError(f'Unsupported  params: {item[0]}')
 
         else:
             raise ValueError(f'Unsupported  params: {item[0]}')
-
+            # print('_MatrixGetMultiItemType')
+            # pass
 
     def to_pkl(self, save_name: str):
         to_pkl(self, save_name)
@@ -983,6 +995,9 @@ class Matrix:
 
         return to_pandas(self, is_matrix_index=is_matrix_index, matrix_index_split=matrix_index_split)
 
+    def to_std_df(self):
+        return to_std_pandas(self)
+
     @property
     def values(self):
         return self.data
@@ -993,7 +1008,7 @@ class Matrix:
         _reverse_dimension = {v: k for k, v in self._dimension_dict.items()}
         columns_str = list(self._axis_name_dict[_reverse_dimension[2]].keys())
         if len(columns_str) > 10:
-            columns_str = str(columns_str[:2])[:-1] + '...' + str(columns_str[:2])[1:]
+            columns_str = str(columns_str[:2])[:-1] + '...' + str(columns_str[-2:])[1:]
         _print_data = "\n             " + str(columns_str)
 
         for i, value in enumerate(self.data):
@@ -1026,11 +1041,31 @@ class Matrix:
         _print_data += _matrix_info + '\n'
         return _print_data
 
+    def __delitem__(self, key):
+        #  del dm[1:5:2]
+        if type(key) == slice:
+            axis_name = np.array(self.axis_name[self._dimension_num_dict[0]])
+            axis_name = np.delete(axis_name, key)
+            self.axis_name[self._dimension_num_dict[0]] = list(axis_name)
+            self.data = np.delete(self.data, key, 0)
+            self._axis_name_to_range()
+
+    # def __delslice__(self, i, j):
+    #
+    #
+    #     pass
+
+    def __len__(self):
+        return self.data.shape
+
     def __str__(self):
         if self.data.size == 0:
             return 'None'
         return self._print()
 
+    def __repr__(self):
+        return self
+
 
 _FillNaType = Union[str, int]
 
@@ -1111,8 +1146,6 @@ def _matrix_fill_na(matrix0, matrix1, fill_na_axis):
                 matrix1 = matrix_fill_na(matrix1, _axis_value, _diff_value)
 
             matrix1.sort(sort_axis_name=_axis_value)
-    # ToDo 还差一步：把补充的数据进行排序
-    # matrix0.sort(axis_name=_axis_value),matrix1.sort(axis_name=_axis_value)
 
     return matrix0, matrix1
 
@@ -1230,14 +1263,8 @@ def read_pandas(obj: pd.DataFrame, shape=None, axis_name: dict = None):
 
 
 def to_pandas(obj: Matrix, is_matrix_index: bool = True, matrix_index_split: str = '.') -> pd.DataFrame:
-    # ToDo 直接将三维转为标准的二维df
+    # 直接将三维转为标准的二维df
     # 检查哪个是一条数据，就能确定
-    # _axis_name_value = list(obj.axis_name.values())
-    # _dimension = 0
-    # for i in range(len(_axis_name_value)):
-    #     if len(_axis_name_value[i]) == 1:
-    #         _dimension = i
-
     new_dm_data = obj.data.reshape([obj.data.shape[0] * obj.data.shape[1], obj.data.shape[2]])
     _index = obj.index
     _row = obj.row
@@ -1262,17 +1289,46 @@ def to_pandas(obj: Matrix, is_matrix_index: bool = True, matrix_index_split: str
     return df
 
 
-def matrix_fillna(arr: np.ndarray):
-    # 目前适用于二维矩阵
+def to_std_pandas(obj: Matrix) -> pd.DataFrame:
+    _axis_name_value = list(obj.axis_name.values())
+    _axis_name_key = list(obj.axis_name.keys())
+    _dimension = -1
+    for i in range(len(_axis_name_value)):
+        if len(_axis_name_value[i]) == 1:
+            _dimension = i
+    if _dimension != -1:
+        if _dimension == 0:
+            _data = obj.values[0, :, :]
+        elif _dimension == 1:
+            _data = obj.values[:, 0, :]
+        elif _dimension == 2:
+            _data = obj.values[:, :, 0]
+        else:
+            raise ValueError(f'Error dimension {_dimension}')
+
+        _axis_name = obj.axis_name
+        _axis_name.pop(_axis_name_key[_dimension])
+        i = 0
+        for key, value in deepcopy(_axis_name).items():
+            if i == 0:
+                _axis_name['index'] = value
+                i += 1
+            else:
+                _axis_name['columns'] = value
+        return DataFrame(_data, _axis_name, _axis_name_key[_dimension]).to_pandas_df()
+
+
+def ffill(arr):
     mask = np.isnan(arr)
     idx = np.where(~mask, np.arange(mask.shape[1]), 0)
     np.maximum.accumulate(idx, axis=1, out=idx)
-    arr[mask] = arr[np.nonzero(mask)[0], idx[mask]]
-    return arr
-
-def to_std_df():
-    # 转为完美的pandas.DataFrame
-    pass
-
+    out = arr[np.arange(idx.shape[0])[:, None], idx]
+    return out
 
 
+def bfill(arr):
+    mask = np.isnan(arr)
+    idx = np.where(~mask, np.arange(mask.shape[1]), mask.shape[1] - 1)
+    idx = np.minimum.accumulate(idx[:, ::-1], axis=1)[:, ::-1]
+    out = arr[np.arange(idx.shape[0])[:, None], idx]
+    return out
